@@ -8,18 +8,20 @@
 #include <stdlib.h>
 #include <inttypes.h>
 #include <string.h>
+#include <strings.h>
 #include <sys/types.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
 #include <time.h>
+#include <stdbool.h>
 
 #include "c.h"
 
 /* initialize a custom exit code for all *_or_err functions */
 extern void strutils_set_exitcode(int exit_code);
 
-extern int parse_size(const char *str, uintmax_t *res, int *power);
+extern int ul_parse_size(const char *str, uintmax_t *res, int *power);
 extern int strtosize(const char *str, uintmax_t *res);
 extern uintmax_t strtosize_or_err(const char *str, const char *errmesg);
 
@@ -27,6 +29,7 @@ extern int ul_strtos64(const char *str, int64_t *num, int base);
 extern int ul_strtou64(const char *str, uint64_t *num, int base);
 extern int ul_strtos32(const char *str, int32_t *num, int base);
 extern int ul_strtou32(const char *str, uint32_t *num, int base);
+extern int ul_strtold(const char *str, long double *num);
 
 extern int64_t str2num_or_err(const char *str, int base, const char *errmesg, int64_t low, int64_t up);
 extern uint64_t str2unum_or_err(const char *str, int base, const char *errmesg, uint64_t up);
@@ -56,6 +59,8 @@ extern void strtotimespec_or_err(const char *str, struct timespec *ts,
 		const char *errmesg);
 extern time_t strtotime_or_err(const char *str, const char *errmesg);
 
+extern bool hyperlinkwanted_or_err(const char *mode, const char *errmesg);
+
 extern int isdigit_strend(const char *str, const char **end);
 #define isdigit_string(_s)	isdigit_strend(_s, NULL)
 
@@ -63,7 +68,7 @@ extern int isxdigit_strend(const char *str, const char **end);
 #define isxdigit_string(_s)	isxdigit_strend(_s, NULL)
 
 
-extern int parse_switch(const char *arg, const char *errmesg, ...);
+extern int ul_parse_switch(const char *arg, const char *errmesg, ...);
 
 #ifndef HAVE_MEMPCPY
 extern void *mempcpy(void *restrict dest, const void *restrict src, size_t n);
@@ -79,15 +84,16 @@ extern char *strnchr(const char *s, size_t maxlen, int c);
 #endif
 
 /* caller guarantees n > 0 */
-static inline void xstrncpy(char *dest, const char *src, size_t n)
+static inline int xstrncpy(char *dest, const char *src, size_t n)
 {
 	size_t len = src ? strlen(src) : 0;
 
 	if (!len)
-		return;
+		return 0;
 	len = min(len, n - 1);
 	memcpy(dest, src, len);
 	dest[len] = 0;
+	return len;
 }
 
 /* This is like strncpy(), but based on memcpy(), so compilers and static
@@ -197,6 +203,36 @@ static inline int strdup_between_offsets(void *stru_dst, void *stru_src, size_t
 #define strdup_between_structs(_dst, _src, _m) \
 		strdup_between_offsets((void *)_dst, (void *)_src, offsetof(__typeof__(*(_src)), _m))
 
+static inline int is_nonnull_offset(const void *stru, size_t offset)
+{
+	const char **o;
+
+	if (!stru)
+		return -EINVAL;
+
+	o = (const char **) ((const char *) stru + offset);
+	return *o != NULL;
+}
+
+#define is_nonnull_member(_stru, _m) \
+		is_nonnull_offset((void *) _stru, offsetof(__typeof__(*(_stru)), _m))
+
+static inline int strcmp_offsets(const void *sa, const void *sb, size_t offset)
+{
+	const char **a = (const char **) ((const char *) sa + offset),
+	           **b = (const char **) ((const char *) sb + offset);
+
+	if (!*a && !*b)
+		return 0;
+	if (!*a)
+		return -1;
+	if (!*b)
+		return 1;
+	return strcmp(*a, *b);
+}
+
+#define strcmp_members(_a, _b, _m) \
+		strcmp_offsets((void *) _a, (void *) _b, offsetof(__typeof__(*(_a)), _m))
 
 extern char *xstrmode(mode_t mode, char *str);
 
@@ -224,7 +260,7 @@ extern int string_to_bitarray(const char *list, char *ary,
 extern int string_to_bitmask(const char *list,
 			     unsigned long *mask,
 			     long (*name2flag)(const char *, size_t));
-extern int parse_range(const char *str, int *lower, int *upper, int def);
+extern int ul_parse_range(const char *str, int *lower, int *upper, int def);
 
 extern int streq_paths(const char *a, const char *b);
 
@@ -356,7 +392,7 @@ static inline size_t __normalize_whitespace(
 		else
 			dst[x++] = src[i++];
 	}
-	if (nsp && x > 0)		/* tailing space */
+	if (nsp && x > 0)		/* trailing space */
 		x--;
 done:
 	dst[x] = '\0';
@@ -388,12 +424,33 @@ static inline void strrem(char *s, int rem)
 	*p = '\0';
 }
 
+/* returns next string after \0 if before @end */
+static inline char *ul_next_string(char *p, char *end)
+{
+	char *last;
+
+	if (!p || !end || p >= end)
+		return NULL;
+
+	for (last = p; p < end; p++) {
+		if (*last == '\0' && p != last)
+			return p;
+		last = p;
+	}
+
+	return NULL;
+}
+
 extern char *strnconcat(const char *s, const char *suffix, size_t b);
 extern char *strconcat(const char *s, const char *suffix);
 extern char *strfconcat(const char *s, const char *format, ...)
 		 __attribute__ ((__format__ (__printf__, 2, 3)));
 
 extern int strappend(char **a, const char *b);
+extern int strfappend(char **a, const char *format, ...)
+		 __attribute__ ((__format__ (__printf__, 2, 3)));
+extern int strvfappend(char **a, const char *format, va_list ap)
+		 __attribute__ ((__format__ (__printf__, 2, 0)));
 
 extern const char *split(const char **state, size_t *l, const char *separator, int quoted);
 
@@ -403,5 +460,6 @@ extern int skip_fline(FILE *fp);
 extern int ul_stralnumcmp(const char *p1, const char *p2);
 
 extern int ul_optstr_next(char **optstr, char **name, size_t *namesz, char **value, size_t *valsz);
+extern int ul_optstr_is_valid(const char *optstr);
 
 #endif
