@@ -1,5 +1,8 @@
 /*
- * Copyright (C) 2011 Karel Zak <kzak@redhat.com>
+ * No copyright is claimed.  This code is in the public domain; do with
+ * it what you wish.
+ *
+ * Written by Karel Zak <kzak@redhat.com> [2011]
  */
 #ifndef UTIL_LINUX_SYSFS_H
 #define UTIL_LINUX_SYSFS_H
@@ -84,6 +87,7 @@ char *sysfs_blkdev_get_devchain(struct path_cxt *pc, char *buf, size_t bufsz);
 int sysfs_blkdev_next_subsystem(struct path_cxt *pc __attribute__((unused)), char *devchain, char **subsys);
 
 int sysfs_blkdev_is_hotpluggable(struct path_cxt *pc);
+int sysfs_blkdev_is_removable(struct path_cxt *pc);
 int sysfs_blkdev_get_wholedisk( struct path_cxt *pc,
                                 char *diskname,
                                 size_t len,
@@ -109,5 +113,14 @@ int sysfs_blkdev_scsi_host_is(struct path_cxt *pc, const char *type);
 int sysfs_blkdev_scsi_has_attribute(struct path_cxt *pc, const char *attr);
 int sysfs_blkdev_scsi_path_contains(struct path_cxt *pc, const char *pattern);
 
+char *sysfs_chrdev_devno_to_devname(dev_t devno, char *buf, size_t bufsiz);
+
+enum sysfs_byteorder {
+	SYSFS_BYTEORDER_LITTLE,
+	SYSFS_BYTEORDER_BIG,
+};
+
+extern enum sysfs_byteorder sysfs_get_byteorder(struct path_cxt *pc);
+extern int sysfs_get_address_bits(struct path_cxt *pc);
 
 #endif /* UTIL_LINUX_SYSFS_H */
