@@ -1,9 +1,9 @@
 /*
- * Copyright (C) 2010 Karel Zak <kzak@redhat.com>
- * Copyright (C) 2010 Davidlohr Bueso <dave@gnu.org>
- *
  * No copyright is claimed.  This code is in the public domain; do with
  * it what you wish.
+ *
+ * Authors: Karel Zak <kzak@redhat.com> [2010]
+ *          Davidlohr Bueso <dave@gnu.org> [2010]
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,7 +64,7 @@ static int do_scale_by_power (uintmax_t *x, int base, int power)
  * Note that the function does not accept numbers with '-' (negative sign)
  * prefix.
  */
-int parse_size(const char *str, uintmax_t *res, int *power)
+int ul_parse_size(const char *str, uintmax_t *res, int *power)
 {
 	const char *p;
 	char *end;
@@ -220,7 +220,7 @@ err:
 
 int strtosize(const char *str, uintmax_t *res)
 {
-	return parse_size(str, res, NULL);
+	return ul_parse_size(str, res, NULL);
 }
 
 int isdigit_strend(const char *str, const char **end)
@@ -247,9 +247,9 @@ int isxdigit_strend(const char *str, const char **end)
 }
 
 /*
- *  parse_switch(argv[i], "on", "off",  "yes", "no",  NULL);
+ *  ul_parse_switch(argv[i], "on", "off",  "yes", "no",  NULL);
  */
-int parse_switch(const char *arg, const char *errmesg, ...)
+int ul_parse_switch(const char *arg, const char *errmesg, ...)
 {
 	const char *a, *b;
 	va_list ap;
@@ -456,21 +456,28 @@ err:
 	errx(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
 }
 
-long double strtold_or_err(const char *str, const char *errmesg)
+int ul_strtold(const char *str, long double *num)
 {
-	double num;
 	char *end = NULL;
 
 	errno = 0;
 	if (str == NULL || *str == '\0')
-		goto err;
-	num = strtold(str, &end);
+		return -(errno = EINVAL);
+	*num = strtold(str, &end);
 
-	if (errno || str == end || (end && *end))
-		goto err;
+	if (errno != 0)
+		return -errno;
+	if (str == end || (end && *end))
+		return -(errno = EINVAL);
+	return 0;
+}
 
-	return num;
-err:
+long double strtold_or_err(const char *str, const char *errmesg)
+{
+	long double num = 0;
+
+	if (ul_strtold(str, &num) == 0)
+		return num;
 	if (errno == ERANGE)
 		err(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
 
@@ -517,6 +524,20 @@ time_t strtotime_or_err(const char *str, const char *errmesg)
 	return (time_t) user_input;
 }
 
+bool hyperlinkwanted_or_err(const char *mode, const char *errmesg)
+{
+	if (mode && strcmp(mode, "never") == 0)
+		return false;
+
+	if (mode && strcmp(mode, "always") == 0)
+		return true;
+
+	if (!mode || strcmp(mode, "auto") == 0)
+		return isatty(STDOUT_FILENO) ? true : false;
+
+	errx(EXIT_FAILURE, "%s: '%s'", errmesg, mode);
+}
+
 /*
  * Converts stat->st_mode to ls(1)-like mode string. The size of "str" must
  * be 11 bytes.
@@ -842,7 +863,7 @@ int string_to_bitmask(const char *list,
  *
  * Returns: 0 on success, <0 on error.
  */
-int parse_range(const char *str, int *lower, int *upper, int def)
+int ul_parse_range(const char *str, int *lower, int *upper, int def)
 {
 	char *end = NULL;
 
@@ -912,7 +933,7 @@ int streq_paths(const char *a, const char *b)
 		if (a_sz + b_sz == 0)
 			return 1;
 
-		/* ignore tailing slash */
+		/* ignore trailing slash */
 		if (a_sz + b_sz == 1 &&
 		    ((a_seg && *a_seg == '/') || (b_seg && *b_seg == '/')))
 			return 1;
@@ -1010,6 +1031,34 @@ int strappend(char **a, const char *b)
 	return 0;
 }
 
+/* the hybrid version of strfconcat and strappend. */
+int strfappend(char **a, const char *format, ...)
+{
+	va_list ap;
+	int res;
+
+	va_start(ap, format);
+	res = strvfappend(a, format, ap);
+	va_end(ap);
+
+	return res;
+}
+
+extern int strvfappend(char **a, const char *format, va_list ap)
+{
+	char *val;
+	int sz;
+	int res;
+
+	sz = vasprintf(&val, format, ap);
+	if (sz < 0)
+		return -errno;
+
+	res = strappend(a, val);
+	free(val);
+	return res;
+}
+
 static size_t strcspn_escaped(const char *s, const char *reject)
 {
         int escaped = 0;
@@ -1182,6 +1231,8 @@ int ul_optstr_next(char **optstr, char **name, size_t *namesz,
 		optstr0++;
 
 	for (p = optstr0; p && *p; p++) {
+		if (!start && *p == '=')
+			return -EINVAL;
 		if (!start)
 			start = p;		/* beginning of the option item */
 		if (*p == '"')
@@ -1217,6 +1268,15 @@ int ul_optstr_next(char **optstr, char **name, size_t *namesz,
 	return 1;				/* end of optstr */
 }
 
+int ul_optstr_is_valid(const char *optstr)
+{
+	int rc;
+	char *p = (char *) optstr;
+
+	while ((rc = ul_optstr_next(&p, NULL, NULL, NULL, NULL)) == 0);
+	return rc < 0 ? 0 : 1;
+}
+
 #ifdef TEST_PROGRAM_STRUTILS
 
 #include "cctype.h"
@@ -1386,6 +1446,33 @@ int main(int argc, char *argv[])
 		printf("\"%s\" --> \"%s\"\n", argv[2], ul_strchr_escaped(argv[2], *argv[3]));
 		return EXIT_SUCCESS;
 
+	} else if (argc == 2 && strcmp(argv[1], "--next-string") == 0) {
+		char *buf = "abc\0Y\0\0xyz\0X";
+		char *end = buf + 12;
+		char *p = buf;
+
+		do {
+			printf("str: '%s'\n", p);
+		} while ((p = ul_next_string(p, end)));
+
+		return EXIT_SUCCESS;
+
+	} else if (argc == 3 && strcmp(argv[1], "--optstr") == 0) {
+
+		size_t namesz, valsz;
+		char *name = NULL, *val = NULL;
+		char *p = argv[2];
+		int rc;
+
+		if (!ul_optstr_is_valid(p))
+			errx(EXIT_FAILURE, _("unsupported option format: %s"), p);
+
+		while ((rc = ul_optstr_next(&p, &name, &namesz, &val, &valsz)) == 0) {
+			printf("'%.*s' : '%.*s'\n", (int) namesz, name,
+						   (int) valsz, val);
+		}
+		if (rc == 1)
+			return EXIT_SUCCESS;
 	} else {
 		fprintf(stderr, "usage: %1$s --size <number>[suffix]\n"
 				"       %1$s --cmp-paths <path> <path>\n"
