@@ -86,11 +86,13 @@ struct libscols_cell {
 	char	*data;
 	size_t	datasiz;
 	char	*color;
+	char	*uri;
 	void    *userdata;
 	int	flags;
 	size_t	width;
 
-	unsigned int is_filled : 1;
+	unsigned int is_filled : 1,
+		     no_uri : 1;
 };
 
 extern int scols_line_move_cells(struct libscols_line *ln, size_t newn, size_t oldn);
@@ -123,6 +125,8 @@ struct libscols_column {
 
 	int	flags;
 	char	*color;		/* default column color */
+	char	*uri;		/* default column URI prefix */
+	struct ul_buffer uri_buf; /* temporary buffer to compose URIs */
 	char	*safechars;	/* do not encode this bytes */
 
 	int (*cmpfunc)(struct libscols_cell *,
@@ -147,7 +151,7 @@ struct libscols_column {
 	void *datafunc_data;
 
 	struct libscols_cell	header;		/* column name with color etc. */
-	char	*shellvar;			/* raw colum name in shell compatible format */
+	char	*shellvar;			/* raw column name in shell compatible format */
 
 	struct list_head	cl_columns;	/* member of table->tb_columns */
 
@@ -265,22 +269,22 @@ struct libscols_table {
 	struct libscols_column *cur_column;	/* currently used column */
 
 	/* flags */
-	unsigned int	ascii		:1,	/* don't use unicode */
-			colors_wanted	:1,	/* enable colors */
-			is_term		:1,	/* isatty() */
-			padding_debug	:1,	/* output visible padding chars */
-			is_dummy_print	:1,	/* printing used for width calculation only */
-			is_shellvar	:1,	/* shell compatible column names */
-			maxout		:1,	/* maximize output */
-			minout		:1,	/* minimize output (mutually exclusive to maxout) */
-			header_repeat   :1,     /* print header after libscols_table->termheight */
-			header_printed  :1,	/* header already printed */
-			priv_symbols	:1,	/* default private symbols */
-			walk_last_done	:1,	/* last tree root walked */
-			no_headings	:1,	/* don't print header */
-			no_encode	:1,	/* don't care about control and non-printable chars */
-			no_linesep	:1,	/* don't print line separator */
-			no_wrap		:1;	/* never wrap lines */
+	bool		ascii	      ,	/* don't use unicode */
+			colors_wanted ,	/* enable colors */
+			is_term	      ,	/* isatty() */
+			padding_debug ,	/* output visible padding chars */
+			is_dummy_print,	/* printing used for width calculation only */
+			is_shellvar   ,	/* shell compatible column names */
+			maxout	      ,	/* maximize output */
+			minout	      ,	/* minimize output (mutually exclusive to maxout) */
+			header_repeat , /* print header after libscols_table->termheight */
+			header_printed,	/* header already printed */
+			priv_symbols  ,	/* default private symbols */
+			walk_last_done,	/* last tree root walked */
+			no_headings   ,	/* don't print header */
+			no_encode     ,	/* don't care about control and non-printable chars */
+			no_linesep    ,	/* don't print line separator */
+			no_wrap	      ;	/* never wrap lines */
 };
 
 #define IS_ITER_FORWARD(_i)	((_i)->direction == SCOLS_ITER_FORWARD)
@@ -501,7 +505,7 @@ enum filter_ntype {
 	F_NODE_EXPR
 };
 
-/* expresion types */
+/* expression types */
 enum filter_etype {
 	F_EXPR_AND,
 	F_EXPR_OR,
