@@ -36,11 +36,12 @@
 #include <sys/ioctl.h>
 
 #include <ctype.h>
+#include <errno.h>
+#include <stdbool.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
+#include <unistd.h>
 #include <getopt.h>
 
 #include "nls.h"
@@ -94,31 +95,152 @@ struct column_control {
 	size_t	nents;		/* number of entries */
 	size_t	maxlength;	/* longest input record (line) */
 	size_t  maxncols;	/* maximal number of input columns */
+	size_t  mincolsep;	/* minimal spaces between columns */
 
-	unsigned int greedy :1,
-		     json :1,
-		     header_repeat :1,
-		     hide_unnamed :1,
-		     maxout : 1,
-		     keep_empty_lines :1,	/* --keep-empty-lines */
-		     tab_noheadings :1;
+	bool	greedy,
+		json,
+		header_repeat,
+		hide_unnamed,
+		maxout : 1,
+		keep_empty_lines,	/* --keep-empty-lines */
+		tab_noheadings,
+		use_spaces;
 };
 
+typedef enum {
+	ANSI_CHR = 'A',
+	ANSI_ESC = 0x1b,
+	ANSI_SGR = '[',
+	ANSI_OSC = ']',
+	ANSI_LNK = '8',
+	ANSI_LBL = 0x7,
+	ANSI_LSP = ';',
+	ANSI_LSG = 'M',
+	ANSI_END = '\\'
+} ansi_esc_states;
+
+/**
+ * Count how many characters are non-printable due to ANSI X3.41 escape codes.
+ *
+ * It detects and count Fe Escape and OSC 8 links sequences. These sequences contains
+ * characters that normally are printable, but due to being part of a escape sequence
+ * are ignored when displayed in console terminals.
+ */
+static inline size_t ansi_esc_width(ansi_esc_states *state, size_t *found, const wchar_t *str, int chw)
+{
+	switch (*state) {
+	case ANSI_CHR:
+		// ANSI X3.41 escape sequences begin with ESC ( written as \x1b \033 or ^[ )
+		if (*str == 0x1b)
+			*state = ANSI_ESC;
+		// Ignore 1 byte C1 control codes (0x80–0x9F) due to conflict with UTF-8 and CP-1252
+		return 0;
+	case ANSI_ESC:
+		// Fe escape sequences allows the range 0x40 to 0x5f
+		switch (*str) {
+		case '[':  // CSI - Control Sequence Introducer
+			*state = ANSI_SGR;
+			break;
+		case ']':  // OSC - Operating System Command
+			*state = ANSI_OSC;
+			break;
+		case '_':  // APC - Application Program Command
+		case 'P':  // DCS - Device Control String
+		case '^':  // PM  - Privacy Message
+			*state = ANSI_END;
+			break;
+		default:
+			*state = ANSI_CHR;
+			return 0;
+		}
+		*found = 1;
+		return 0;
+	case ANSI_SGR:
+		*found += chw;
+		// Fe escape sequences allows the range 0x30-0x3f
+		// However SGR (Select Graphic Rendition) only uses: 0-9 ';' ':'
+		if (*str >= '0' && *str <= '?')
+			return 0;
+		// Fe ends with the range 0x40-0x7e but SGR ends with 'm'
+		if (*str <= '@' && *str >= '~')
+			*found = 0;
+		break;
+	case ANSI_OSC:
+		*found += chw;
+		if (*str == ANSI_LNK) // OSC8-Link
+			*state = ANSI_LNK; 
+		else
+			*state = ANSI_END; // other command sequences are ignored
+		return 0;
+	case ANSI_LNK: // OSC8 Terminal Hiperlink Sequence
+		switch (*str) {
+		case 0x7:  // Separated by BEL
+			*state = ANSI_LBL; //#  \e]8;;LINK\aTEXT\e]8;;\a  #
+			break;
+		case 0x1b: // OSC8-Link separated by ESC-BACKSLASH
+			*found += 2;
+			*state = ANSI_LBL; //#  \e]8;;LINK\e\\TEXT\e]8;;\e\\  #
+			break;
+		default:
+			*found += chw;
+		}
+		return 0; // ignore link width
+	case ANSI_LBL:
+		if (*str == 0x1b) { // Link label goes until ESC BACKSLASH
+			*found += chw;
+			*state = ANSI_LSP;
+		}
+		return 0;
+	case ANSI_LSP:
+		*found += chw;
+		if (*str == '[') // SGR FG/BG colors nested inside OSC8-Link sequence
+			*state = ANSI_LSG;
+		else
+			*state = ANSI_END; //# Link label ends with \e[8;;\e\\ #
+		return 0;
+	case ANSI_LSG: //#  \e]8;;LINK\e\\\e[1;34mTEXT\e[0m\e]8;;\e\\  #
+		*found += chw;
+		if (*str < '0' || *str > '?') //  SGR color sequence ends with 'm'
+			*state = ANSI_LBL;
+		return 0;
+	case ANSI_END:
+		switch (*str) {
+		case 0x1b:  // APC/OSC8-Links ends with ESC-BACKSLASH
+			*found += chw;
+			break;
+		case 0x7:  // APC/OSC/OSC8-Links ends with BEL
+#ifdef HAVE_WIDECHAR
+		case 0x9c:  // APC/DCS/DM ends with ST (String Terminator)
+#endif
+			break;
+		default:
+			*found += chw;
+		return 0;
+	}
+	}
+	size_t res = *found;
+	*state = ANSI_CHR;
+	*found = 0;
+	return res;
+}
+
 static size_t width(const wchar_t *str)
 {
-	size_t width = 0;
+	size_t count = 0;
+	size_t found = 0;
+	ansi_esc_states state = ANSI_CHR;
 
 	for (; *str != '\0'; str++) {
 #ifdef HAVE_WIDECHAR
 		int x = wcwidth(*str);	/* don't use wcswidth(), need to ignore non-printable */
-		if (x > 0)
-			width += x;
 #else
-		if (isprint(*str))
-			width++;
+		int x = isprint(*str) ? 1 : 0;
 #endif
+		int chw = x > 0 ? x : 0;
+		size_t nonpr = ansi_esc_width(&state, &found, str, chw);
+		count += chw - nonpr;
 	}
-	return width;
+	return count;
 }
 
 static wchar_t *mbs_to_wcs(const char *s)
@@ -360,7 +482,7 @@ static void apply_columnflag_from_list(struct column_control *ctl, const char *l
 		}
 
 		/* parse range (N-M) */
-		if (strchr(*one, '-') && parse_range(*one, &low, &up, 0) == 0) {
+		if (strchr(*one, '-') && ul_parse_range(*one, &low, &up, 0) == 0) {
 			for (; low <= up; low++) {
 				if (low < 0)
 					cl = get_last_visible_column(ctl, (low * -1) -1);
@@ -585,7 +707,7 @@ static void add_entry(struct column_control *ctl, size_t *maxents, wchar_t *wcs)
 {
 	if (ctl->nents <= *maxents) {
 		*maxents += 1000;
-		ctl->ents = xrealloc(ctl->ents, *maxents * sizeof(wchar_t *));
+		ctl->ents = xreallocarray(ctl->ents, *maxents, sizeof(wchar_t *));
 	}
 	ctl->ents[ctl->nents] = wcs;
 	ctl->nents++;
@@ -662,17 +784,25 @@ static int read_input(struct column_control *ctl, FILE *fp)
 		}
 	} while (rc == 0);
 
+	free(buf);
+
 	return rc;
 }
 
 
 static void columnate_fillrows(struct column_control *ctl)
 {
-	size_t chcnt, col, cnt, endcol, numcols;
+	size_t chcnt, col, cnt, endcol, numcols, remains;
 	wchar_t **lp;
 
-	ctl->maxlength = (ctl->maxlength + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1);
+	if (ctl->use_spaces)
+		ctl->maxlength += ctl->mincolsep;
+	else
+		ctl->maxlength = (ctl->maxlength + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1);
 	numcols = ctl->termwidth / ctl->maxlength;
+	remains = ctl->termwidth % ctl->maxlength;
+	if (ctl->use_spaces && remains + ctl->mincolsep >= ctl->maxlength)
+		numcols++;
 	endcol = ctl->maxlength;
 	for (chcnt = col = 0, lp = ctl->ents; /* nothing */; ++lp) {
 		fputws(*lp, stdout);
@@ -684,9 +814,16 @@ static void columnate_fillrows(struct column_control *ctl)
 			endcol = ctl->maxlength;
 			putwchar('\n');
 		} else {
-			while ((cnt = ((chcnt + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1))) <= endcol) {
-				putwchar('\t');
-				chcnt = cnt;
+			if (ctl->use_spaces) {
+				while (chcnt < endcol) {
+					putwchar(' ');
+					chcnt++;
+				}
+			} else {
+				while ((cnt = ((chcnt + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1))) <= endcol) {
+					putwchar('\t');
+					chcnt = cnt;
+				}
 			}
 			endcol += ctl->maxlength;
 		}
@@ -697,12 +834,18 @@ static void columnate_fillrows(struct column_control *ctl)
 
 static void columnate_fillcols(struct column_control *ctl)
 {
-	size_t base, chcnt, cnt, col, endcol, numcols, numrows, row;
+	size_t base, chcnt, cnt, col, endcol, numcols, numrows, row, remains;
 
-	ctl->maxlength = (ctl->maxlength + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1);
+	if (ctl->use_spaces)
+		ctl->maxlength += ctl->mincolsep;
+	else
+		ctl->maxlength = (ctl->maxlength + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1);
 	numcols = ctl->termwidth / ctl->maxlength;
+	remains = ctl->termwidth % ctl->maxlength;
 	if (!numcols)
 		numcols = 1;
+	if (ctl->use_spaces && remains + ctl->mincolsep >= ctl->maxlength)
+		numcols++;
 	numrows = ctl->nents / numcols;
 	if (ctl->nents % numcols)
 		++numrows;
@@ -714,9 +857,16 @@ static void columnate_fillcols(struct column_control *ctl)
 			chcnt += width(ctl->ents[base]);
 			if ((base += numrows) >= ctl->nents)
 				break;
-			while ((cnt = ((chcnt + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1))) <= endcol) {
-				putwchar('\t');
-				chcnt = cnt;
+			if (ctl->use_spaces) {
+				while (chcnt < endcol) {
+					putwchar(' ');
+					chcnt++;
+				}
+			} else {
+				while ((cnt = ((chcnt + TABCHAR_CELLS) & ~(TABCHAR_CELLS - 1))) <= endcol) {
+					putwchar('\t');
+					chcnt = cnt;
+				}
 			}
 			endcol += ctl->maxlength;
 		}
@@ -773,11 +923,12 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -o, --output-separator <string>  columns separator for table output (default is two spaces)\n"), out);
 	fputs(_(" -s, --separator <string>         possible table delimiters\n"), out);
 	fputs(_(" -x, --fillrows                   fill rows before columns\n"), out);
+	fputs(_(" -S, --use-spaces <number>        minimal whitespaces between columns (no tabs)\n"), out);
 
 
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(34));
-	printf(USAGE_MAN_TAIL("column(1)"));
+	fprintf(out, USAGE_HELP_OPTIONS(34));
+	fprintf(out, USAGE_MAN_TAIL("column(1)"));
 
 	exit(EXIT_SUCCESS);
 }
@@ -821,6 +972,7 @@ int main(int argc, char **argv)
 		{ "tree",                required_argument, NULL, 'r' },
 		{ "tree-id",             required_argument, NULL, 'i' },
 		{ "tree-parent",         required_argument, NULL, 'p' },
+		{ "use-spaces",          required_argument, NULL, 'S' },
 		{ "version",             no_argument,       NULL, 'V' },
 		{ NULL,	0, NULL, 0 },
 	};
@@ -840,7 +992,7 @@ int main(int argc, char **argv)
 	ctl.output_separator = "  ";
 	ctl.input_separator = mbs_to_wcs("\t ");
 
-	while ((c = getopt_long(argc, argv, "C:c:dE:eH:hi:Jl:LN:n:mO:o:p:R:r:s:T:tVW:x", longopts, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "C:c:dE:eH:hi:Jl:LN:n:mO:o:p:R:r:S:s:T:tVW:x", longopts, NULL)) != -1) {
 
 		err_exclusive_options(c, longopts, excl, excl_st);
 
@@ -907,11 +1059,15 @@ int main(int argc, char **argv)
 		case 'r':
 			ctl.tree = optarg;
 			break;
+		case 'S':
+			ctl.use_spaces = 1;
+			ctl.mincolsep = strtou32_or_err(optarg, _("invalid spaces argument"));
+			break;
 		case 's':
 			free(ctl.input_separator);
 			ctl.input_separator = mbs_to_wcs(optarg);
 			if (!ctl.input_separator)
-				err(EXIT_FAILURE, _("failed to use input separator"));
+				err(EXIT_FAILURE, _("failed to parse input separator"));
 			ctl.greedy = 0;
 			break;
 		case 'T':
